'''
Common settings for camera_sever.py (Python2) and camera_client.py (Python3)
'''

# Hostname (or IP address) of the server. This is the address that the client
# tries to connect to
SERVER_HOSTNAME = '127.0.0.1'

PORT = 50075

# If for some reason the default port is in use / does not work,
# try in order some other from this list
# FIXME NOT IMPLEMENTED YET
# ALTERNATIVE_PORTS = list(range(50071, 50080))



SAVING_DRIVE = 'D:\\'



'''
Camera client software.

Meant to be running using Python 3.
'''

import socket
import time
import os
import subprocess
import platform

from gonioimsoft.directories import CODE_ROOTDIR
import gonioimsoft.camera_communication as cac

MAX_RETRIES = 100
RETRY_INTERVAL = 1

class CameraClient:
    '''
    Connecting to the CameraServer and sending imaging commands.

    No data is transmitted over the socket connection, only commands (strings).
    It's CameraServer's job to store the images.

    Attributes
    -----------
    python2 : string
        In path command to open Python 2. If empty string, use
        defaults "C:\Python27\python.exe" (Windows) or "python2" (other)
    '''
    def __init__(self):
        '''
        Initialization of the CameraClient 
        '''
        self.host = cac.SERVER_HOSTNAME
        self.port = cac.PORT
        
        self._python2 = ''
    
    @property
    def python2(self):
        if self._python2:
            cmd = self._python2
        else:
            if platform.system() == 'Windows':
                cmd = 'C:\Python27\python.exe'
            else:
                cmd = 'python2'
        return cmd 

    @python2.setter
    def python2(self, string):
        self._python2 = string


    def sendCommand(self, command_string, retries=MAX_RETRIES):
        '''
        Send an arbitrary command to the CameraServer.
        All the methods of the Camera class (see camera_server.py) are supported.

        INPUT ARGUMETNS     DESCRIPTION
        command_string      function;parameters,comma,separated
                            For example "acquireSeries;0,01,0,5,'label'"

        This is where a socket connection to the server is formed. After the command_string
        has been send, the socket terminates.
        '''

        tries = 0
        
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            
            while True:
                try:
                    s.connect((self.host, self.port))
                    break
                except ConnectionRefusedError:
                    tries += 1
                    if tries > retries:
                        raise ConnectionRefusedError('Cannot connect to the camera server')
                    print('Camera server connection refused, retrying...')
                    time.sleep(RETRY_INTERVAL)
                
            s.sendall(command_string.encode())


    def acquireSeries(self, exposure_time, image_interval, N_frames, label, subdir, trigger_direction):
        '''
        Acquire a time series of images.
        For more see camera_server.py.

        Notice that it is important to give a new label every time
        or to change data savedir, otherwise images may be written over
        each other (or error raised).
        '''
        function = 'acquireSeries;'
        parameters = "{}:{}:{}:{}:{}:{}".format(exposure_time, image_interval, N_frames, label, subdir, trigger_direction)
        message = function+parameters
        
        self.sendCommand(message)


    def acquireSingle(self, save, subdir):
        self.sendCommand('acquireSingle;{}:{}'.format(str(save), subdir))

    
    def setSavingDirectory(self, saving_directory):
        self.sendCommand('setSavingDirectory;'+saving_directory)


    def saveDescription(self, filename, string):
        self.sendCommand('saveDescription;'+filename+':'+string)

    def set_roi(self, roi):
        self.sendCommand('set_roi;{}:{}:{}:{}'.format(*roi))

    def set_save_stack(self, boolean):
        self.sendCommand('set_save_stack;{}'.format(boolean))

    def isServerRunning(self):
        try:
            self.sendCommand('ping;Client wants to know if server is running', retries=0)
        except ConnectionRefusedError:
            return False
        return True


    def startServer(self):
        '''
        Start a local camera server instance.
        '''

        subprocess.Popen([self.python2, os.path.join(CODE_ROOTDIR, 'camera_server.py')],
            stdout=open(os.devnull, 'w'))


    def close_server(self):
        '''
        Sends an exit message to the server, to which the server should respond
        by closing itself down.
        '''
        try:
            self.sendCommand('exit;'+'None', retries=0)
        except ConnectionRefusedError:
            pass
        


def test():
    cam = CameraClient()
    cam.acquireSeries(0.01, 0, 5, 'test')


if __name__ == "__main__":
    test()




import os
import copy
import platform
import string
import time
import json


OS = platform.system()
if OS == 'Windows':
    import msvcrt
else:
    import sys

from gonioimsoft.version import __version__
from gonioimsoft.directories import PUPILDIR
import gonioimsoft.core as core
from gonioimsoft.imaging_parameters import (
        DEFAULT_DYNAMIC_PARAMETERS,
        ParameterEditor,
        )


help_string = """List of commands and their options\n
GENERAL
 help                       Prints this message
 suffix [SUFFIX]            Add a suffix SUFFIX to saved image folders
MOTORS
 where [i_motor]            Prints the coordinates of motor that has index i_motor
 drive [i_motor] [pos]      Drive i_motor to coordinates (float)


"""

help_limit = """Usage of limit command
limit []"""


class Console:
    '''
    Operation console for TUI or other user interfaces.

    Capabilities:
    - changing imaging parameters
    - setting save suffix
    - controlling motors and setting their limits
    
    In tui, this console can be opened by pressing ` (the keyboard button next to 1)
    '''
    def __init__(self, core_dynamic):
        '''
        core_dynamic        An instance of core.Dynamic class.
        '''
        self.dynamic = core_dynamic


    def enter(self, command):
        '''
        Calling a command 
        '''
        command_name = command.split(' ')[0]
        args = command.split(' ')[1:]

        if hasattr(self, command_name):
            method = getattr(self, command_name)
            try:
                method(*args)
            except TypeError as e:
                print(e)
                self.help()
                
        else:
            print('Command {} does not exit'.format(command_name))
            self.help()
    

    def help(self):
        '''
        Print the help string on screen.
        '''
        print(help_string)
    

    def suffix(self, suffix):
        '''
        Set suffix to the image folders being saved
        '''
        # Replaces spaces by underscores
        if ' ' in suffix:
            suffix = suffix.replace(' ', '_')
            print('Info: Replaced spaces in the suffix with underscores')
        
        # Replace illegal characters by x
        legal_suffix = ""
        for letter in suffix:
            if letter in string.ascii_letters+'_()-'+'0123456789.':
                legal_suffix += letter
            else:
                print('Replacing illegal character {} with x'.format(letter))
                legal_suffix += 'x'
        
        print('Setting suffix {}'.format(legal_suffix))
        self.dynamic.set_subfolder_suffix(legal_suffix)


    def limitset(self, side, i_motor):
        '''
        Sets the current position as a limit.

        action      "set" or "get"
        side        "upper" or "lower"
        i_motor     0, 1, 2, ...
        '''
        
        if side == 'upper':
            self.dynamic.motors[i_motor].set_upper_limit()
        elif side == 'lower':
            self.dynamic.motors[i_motor].set_lower_limit()
   

    def limitget(self, i_motor):
        '''
        Gets the current limits of a motor
        '''
        mlim = self.dynamic.motors[i_motor].get_limits()
        print('  Motor {} limited at {} lower and {} upper'.format(i_motor, *mlim))


    def where(self, i_motor):
        # Getting motor's position
        mpos = self.dynamic.motors[motor].get_position()
        print('  Motor {} at {}'.format(motor, mpos))


    def drive(self, i_motor, position):
        self.dynamic.motors[i_motor].move_to(position)
        

    def macro(self, command, macro_name):
        '''
        Running and setting macros (automated imaging sequences.)
        '''
        if command == 'run':
            self.dynamic.run_macro(macro_name)
        elif command == 'list':

            print('Following macros are available')
            for line in self.dynamic.list_macros():
                print(line)

        elif command == 'stop':
            for motor in self.dynamic.motors:
                motor.stop()


    def set_roi(self, x,y,w,h):
        self.dynamic.camera.set_roi( (x,y,w,h) )


    def eternal_repeat(self, isi):

        isi = float(isi)
        print(isi)
        
        suffix = "eternal_repeat_isi{}s".format(isi)
        suffix = suffix + "_rep{}"
        i_repeat = 0
        
        while True:
            self.suffix(suffix.format(i_repeat))

            start_time = time.time()
            
            if self.dynamic.image_series(inter_loop_callback=self.image_series_callback) == False:
                break
            i_repeat += 1

            sleep_time = isi - float(time.time() - start_time)
            if sleep_time > 0:
                time.sleep(sleep_time)


    def chain_presets(self, delay, *preset_names):
        '''
        Running multiple presets all one after each other,
        in a fixed (horizonta, vertical) location.

        delay       In seconds, how long to wait between presets
        '''
        delay = float(delay)
        original_parameters = copy.copy(self.dynamic.dynamic_parameters)

        
        print('Repeating presets {}'.format(preset_names))
        for preset_name in preset_names:
            print('Preset {}'.format(preset_name))
            
            self.dynamic.load_preset(preset_name)
            
            if self.dynamic.image_series(inter_loop_callback=self.image_series_callback) == False:
                break

            time.sleep(delay)

        print('Finished repeating presets')
        self.dynamic.dynamic_parameters = original_parameters

            
    def set_rotation(self, horizontal, vertical):
        ho = int(horizontal)
        ve = int(vertical)
        cho, cve = self.dynamic.reader.latest_angle
        
        self.dynamic.reader.offset = (cho-ho, cve-ve)



class TextUI:
    '''
    A simple text based user interface goniometric imaging.

    Attrubutes
    ----------
    console : object
    choices : dict
        Main menu choices
    quit : bool
        If changes to True, quit.
    expfn : string
        Filename of the experiments.json file
    glofn : string
        Filename of the locked parameters setting name

    '''
    def __init__(self):
        self.dynamic = core.Dynamic()
        
        # Get experimenters list or if not present, use default
        self.expfn = os.path.join(PUPILDIR, 'experimenters.json')
        if os.path.exists(self.expfn):
            try:
                with open(self.expfn, 'r') as fp: self.experimenters = json.load(fp)
            except:
                self.experimenters = ['gonioims']
        else:
            self.experimenters = ['gonioims']
        

        # Get locked parameters
        self.glofn = os.path.join(PUPILDIR, 'locked_parameters.json')
        if os.path.exists(self.glofn):
            try:
                 with open(self.glofn, 'r') as fp: self.locked_parameters = json.load(fp)
            except:
                self.locked_parameters = {}
        else:
            self.locked_parameters = {}
            

   
        self.choices = [['Static imaging', self.loop_static],
                ['Dynamic imaging', self.loop_dynamic],
                ['Trigger only (external software for camera)', self.loop_trigger],
                ['', None],
                ['Edit locked parameters', self.locked_parameters_edit],
                ['', None],
                ['Quit', self.quit],
                ['', None],
                ['Start camera server (local)', self.dynamic.camera.startServer],
                ['Stop camera server', self.dynamic.camera.close_server],
                ['Set Python2 command (current {})', self.set_python2]]


        self.quit = False

        self.console = Console(self.dynamic)
        self.console.image_series_callback = self.image_series_callback


    @property
    def menutext(self):

        # Check camera server status
        if self.dynamic.camera.isServerRunning():
            cs = 'ON'
        else:
            cs = 'OFF'

        # Check serial (Arduino) status
        ser = self.dynamic.reader.serial
        if ser is None:
            ar = 'Serial UNAVAIBLE'
        else:
            if ser.is_open:
                ar = 'Serial OPEN ({} @{} Bd)'.format(ser.port)
            else:
                ar = 'Serial CLOSED'

        # Check DAQ
        if core.nidaqmx is None:
            daq = 'UNAVAILABLE'
        else:
            daq = 'AVAILABLE'

        status = "\n CamServer {} | {} | nidaqmx {}".format(cs, ar, daq)
        
        menutext = "Pupil Imsoft - Version {}".format(__version__)
        menutext += "\n" + max(len(menutext), len(status)) * "-"
        menutext += status
        return menutext + "\n"


    @staticmethod
    def _readKey():
        if OS == 'Windows':
            if msvcrt.kbhit():
                key = ord(msvcrt.getwch())
                return chr(key)
            return ''
        else:
            return sys.stdin.read(1)


    @staticmethod
    def _clearScreen():
        if os.name == 'posix':
            os.system('clear')
        elif os.name == 'nt':
            os.system('cls')


    @staticmethod
    def _print_lines(lines):
        
        for text in lines:
            print(text)
        

    def _selectItem(self, items):
        '''
        Select an item from a list.
        
        Empty string items are converted to a space
        '''
        real_items = []
        i = 0
        for item in items:
            if item != '':
                print('{}) {}'.format(i+1, item))
                real_items.append(item)
                i += 1
            else:
                print()

        selection = ''
        while True:
            new_char = self._readKey()
            if new_char:
                selection += new_char
                print(selection)
            if selection.endswith('\r') or selection.endswith('\n'):
                try:
                    selection = int(selection)
                    real_items[selection-1]
                    break
                except ValueError:
                    print('Invalid input')
                    selection = ''
                except IndexError:
                    print('Invalid input')
                    selection = ''
        return real_items[selection-1]
    
    def set_python2(self):
        print('Current Python2 command: {}'.format(self.dynamic.camera.python2))
        sel = input('Change (yes/no)').lower()
        if sel == 'yes':
            newp = input('>>')
            print('Chaning...')
            self.dynamic.camera.python2 = newp
            input('press enter to continue')
        else:
            print('No changes!')

    def loop_trigger(self):
        '''
        Simply NI trigger when change in rotatory encoders, leaving camera control
        to an external software (the original loop static).
        
        Space to toggle triggering.
        '''
        self.loop_dynamic(static=True, camera=False)
                

    def loop_static(self):
        '''
        Running the static imaging protocol.
        '''
        self.loop_dynamic(static=True)
        

    def image_series_callback(self, label, i_repeat):
        '''
        Callback passed to image_series
        '''
        if label:
            print(label)
        
        key = self._readKey()

        if key == '\r':
            # If Enter presed return False, stopping the imaging
            print('User pressed enter, stopping imaging')
            return False
        else:
            return True


    def loop_dynamic(self, static=False, camera=True):
        '''
        Running the dynamic imaging protocol.

        static : bool
            If False, run normal imaging where pressing space runs the imaging protocol.
            If True, run imaging when change in rotary encoders (space-key toggled)
        camera : bool
            If True, control camera.
            If False, assume that external program is controlling the camera, and send trigger
        '''
        trigger = False
        
        self.dynamic.locked_parameters = self.locked_parameters
        
        self.dynamic.set_savedir(os.path.join('imaging_data_'+self.experimenter), camera=camera)
        name = input('Name ({})>> '.format(self.dynamic.preparation['name']))
        sex = input('Sex ({})>> '.format(self.dynamic.preparation['sex']))
        age = input('Age ({})>> '.format(self.dynamic.preparation['age']))
        self.dynamic.initialize(name, sex, age, camera=camera)

        upper_lines = ['-','Dynamic imaging', '-', 'Help F1', 'Space ']

        while True:
            
            lines = upper_lines

            key = self._readKey()

            if static:
                if trigger and self.dynamic.trigger_rotation:
                    if camera:
                        self.dynamic.image_series(inter_loop_callback=self.image_series_callback)
                    else:
                        self.dynamic.send_trigger()
                if key == ' ':
                    trigger = not trigger
                    print('Rotation triggering now set to {}'.format(trigger))
            else:
                if key == ' ':
                    if camera:
                        self.dynamic.image_series(inter_loop_callback=self.image_series_callback)
                    else:
                        self.dynamic.send_trigger()
            
            if key == 112:
                lines.append('')
            elif key == '0':
                self.dynamic.set_zero()
            elif key == 's':
                if camera:
                    self.dynamic.take_snap(save=True)
            elif key == '\r':
                # If user hits enter we'll exit
                break

            elif key == '[':
                self.dynamic.motors[0].move_raw(-1)
            elif key == ']':
                self.dynamic.motors[0].move_raw(1)
            
            elif key == 'o':
                self.dynamic.motors[1].move_raw(-1)
            elif key == 'p':
                self.dynamic.motors[1].move_raw(1)

            elif key == 'l':
                self.dynamic.motors[2].move_raw(-1)
            elif key == ';':
                self.dynamic.motors[2].move_raw(1)

            elif key == '`':
                user_input = input("Type command >> ")
                self.console.enter(user_input)

            elif key == '' and not (static and self.dynamic.trigger_rotation):
                if camera:
                    # When there's no input just update the live feed
                    self.dynamic.take_snap(save=False)
            
            
            #self._clearScreen()
            #self._print_lines(lines)

            self.dynamic.tick()

        self.dynamic.finalize()


    def run(self):
        '''
        Run TUI until user quitting.
        '''
        # Check if userdata directory settings exists
        if not os.path.isdir(PUPILDIR):
            print('\nFIRST RUN NOTICE\n------------------')
            print(('Pupil Imsoft needs a location where '
                'to save user files\n  - list of experimenters\n  - settings'
                '\n  - created protocol files'))
            print('This is not the location where imaging data gets saved (no big files)')
            print('\nCreate {}'.format(PUPILDIR))

            while True:
                sel = input ('(yes/no) >> ').lower()
                if sel == 'yes':
                    os.makedirs(PUPILDIR)
                    print('Sucess!')
                    time.sleep(2)
                    break
                elif sel == 'no':
                    print('Warning! Cannot save any changes')
                    time.sleep(2)
                    break
                else:
                    print('Whaat? Please try again')
                    time.sleep(1)
            
        
        self._clearScreen()
        
        print(self.menutext)
        
        print('Select experimenter\n--------------------')
        while True:
            extra_options = [' (Add new)', ' (Remove old)', ' (Save current list)']
            experimenter = self._selectItem(self.experimenters+extra_options).lower()
            
            # Select operation
            if experimenter == '(add new)':
                name = input('Name >>')
                self.experimenters.append(name)

            elif experimenter == '(remove old)':
                print('Select who to remove (data remains)')
                name = self._selectItem(self.experimenters+['..back (no deletion)'])

                if name in self.experimenters:
                    self.experimenters.pop(self.experimenters.index(name))
            elif experimenter == '(save current list)':
                if os.path.isdir(PUPILDIR):
                    with open(self.expfn, 'w') as fp: json.dump(self.experimenters, fp)
                    print('Saved!')
                else:
                    print('Saving failed (no {})'.format(PUPILDIR))
                time.sleep(2)
            else:
                # Got a name
                break

            self._clearScreen()

        self.experimenter = experimenter
        self._clearScreen()

        self.quit = False
        while not self.quit:
            print(self.menutext)
            
            menuitems = [x[0] for x in self.choices]
            menuitems[-1] = menuitems[-1].format(self.dynamic.camera.python2)

            selection = self._selectItem(menuitems)
            self.choices[menuitems.index(selection)][1]()
            
            time.sleep(1)

            self._clearScreen()

        self.dynamic.exit()
        time.sleep(1)


    def locked_parameters_edit(self):
        
        while True:
            self._clearScreen()
            print(self.menutext)
            print('Here, any of the imaging parameters can be made locked,')
            print('overriding any presets/values setat imaging time.')
            
            print('\nCurrent locked are')
            if not self.locked_parameters:
                print('  (NONE)')
            for name in self.locked_parameters:
                print('  {}'.format(name))
            print()

            sel = self._selectItem(['Add locked', 'Remove locked', 'Modify values', '.. back (and save)'])
            
            if sel == 'Add locked':
                choices = list(DEFAULT_DYNAMIC_PARAMETERS.keys())
                sel2 = self._selectItem(choices+[' ..back'])
                
                if sel2 in choices:
                    self.locked_parameters[sel2] = DEFAULT_DYNAMIC_PARAMETERS[sel2]
            elif sel == 'Remove locked':
                choices = list(self.locked_parameters.keys())
                sel2 = self._selectItem(choices+[' ..back'])
                
                if sel2 in choices:
                    del self.locked_parameters[sel2]

            elif sel == 'Modify values':
                self.locked_parameters = ParameterEditor(self.locked_parameters).getModified()

            elif sel == '.. back (and save)':
                if os.path.isdir(PUPILDIR):
                    with open(self.glofn, 'w') as fp: json.dump(self.locked_parameters, fp)
                break


    def quit(self):
        self.quit = True



def main():
    tui = TextUI()
    tui.run()

if __name__ == "__main__":
    main()




import os
import ast

def list_macros():
    '''
    Lists all available macros by names that can be
    directly passed to load function in this module.
    '''

    return [fn[:-4] for fn in os.listdir('macros') if fn.endswith('.txt')]

def load(macro_name):
    '''
    Loads a macro from text file.

    macro_name is the name of a macro file inside macros folder,
    without the file ending.

    Loaded macro that is returned is a list whose items
    are macro commands understood by DynamicImaging in core.py
    '''

    macro = []

    with open(os.path.join('macros', macro_name+'.txt'), 'r') as fp:
        for line in fp:
            if line:
                macro.append(ast.literal_eval(line))
    
    return macro

def save(macro_name, macro):
    with open(os.path.join('macros', macro_name+'.txt'), 'w') as fp:
        for line in macro:
            fp.write(str(line)+'\n')





import math
import time
import atexit
import threading

from gonioimsoft.anglepairs import degrees2steps

class Motor:
    '''
    Moving motors with limits.
    '''

    def __init__(self, ArduinoReader, i_motor, i_sensor):
        '''
        ArduinoReader
        i_motor             Index number of the motor
        #i_sensor            None or index number of the sensor
        '''
        self.reader = ArduinoReader
        self.i_motor = i_motor
        self.i_sensor = i_sensor
        
        # If no sensor is connected with the motor (i_sensor == None),
        # at least we keep track how many times have we moved.
        self.position = 0
        
        self.limits = [-math.inf, math.inf]
        
        # Moving motor specific place using a sensor
        # maxmis = Maximum allowed error when using move_to
        self.maxmis = 6
        self.thread = None
        self._stop = False
        
        atexit.register(self.move_to, 0)


    def get_position(self):
        '''
        Returns the current position of the motor
        '''
        if self.i_sensor is None:
            return self.position
        else:
            return self.reader.get_sensor(self.i_sensor)

    def move_raw(self, direction, time=1):
        '''
        Return False if movement wans't allowed, otherwise True.
        '''
        curpos = self.get_position()

        # Only move so that we don't go over limits
        if ((self.limits[0] <= curpos and direction >= 0) or
                (curpos <= self.limits[1] and direction < 0) or
                (self.limits[0] < curpos < self.limits[1])):
            
            self.reader.move_motor(self.i_motor, direction, time=time)
            self.position += time*direction

            return True
        else:
            return False

    
    def move_to(self, motor_position):
        '''
        Move motor to specific position.

        If self.i_sensor == None:
            motor_position is measured in time units, hoping that the motor
            has constant speed

        otherwise
            motor_position is measured in degrees from the zero position
        '''
        print('Driving motor {} to {}'.format(self.i_motor, motor_position))
        if self.i_sensor is None:
            # If no extra sensor connected to the motor we'll just move
            # based on where we think we are
            position = self.get_position()
            time = position - motor_position
            if time >= 0:
                direction = 1
            else:
                direction = -1
                time = -time
            self.move_raw(direction, time=time)
        else:
            # If we have a sensor we should move towards it, launch a thread
            # that runs in background until the task finished

            if not self.thread:
                callable_getpos = lambda: self.reader.get_sensor(self.i_sensor)
            
                self.thread = threading.Thread(target=self._move_to_thread,
                                               args=(degrees2steps(motor_position), callable_getpos))
                self._stop = False
                self.thread.start()
                print('started thread')


    def _move_to_thread(self, target, callable_getpos):
        '''
        This is a target
        
        callable_getpos         A callable that returns the current position of the 
        '''
        print('got to thread')
        
        while not self._stop:

            pos = callable_getpos()

            if target-self.maxmis/2 < pos < target+self.maxmis/2:
                break

            direction = pos-target
            
            if not self.move_raw(direction, time=0.1):
                # If hitting the limits stop here
                break
            
            # The thread can sleep 95 ms while waiting the motor to move
            time.sleep(0.095)

        self.thread = None

    def stop(self):
        self._stop = True

    def reached_target(self):
        '''
        Returns True if the motor has reached its target set at move_to.
        '''
        if self.thread:
            return False
        else:
            return True

    def set_upper_limit(self):
        '''
        Sets current position as the upper limit
        '''
        self.limits[1] = self.position


    def set_lower_limit(self):
        '''
        Sets current position as the lower limit
        '''
        self.limits[0] = self.position

    def get_limits(self):
        return self.limits





'''
Default dynamic parameters and ParameterEditor for letting the user
modify them in the program.
'''
import os
import time
import ast
import json

from gonioimsoft.directories import PUPILDIR


DEFAULT_DYNAMIC_PARAMETERS = {'isi': 10.0, 'repeats': 1, 'pre_stim': 0.000,
        'stim': 0.200, 'post_stim': 0.00, 'frame_length' : 0.010,
        'ir_imaging': 5, 'ir_waiting': 0, 'ir_livefeed': 1,
        'flash_on': 8, 'flash_off': 0,
        'ir_channel': "Dev1/ao1", 'flash_channel': "Dev1/ao0",
        'suffix': '', 'trigger_channel': "/Dev1/PFI0",
        'trigger_out_channel': "Dev2/ao0",
        'biosyst_stimulus': '',
        'biosyst_channel': 2,
        'avgint_adaptation': 0,
        'flash_type': 'square',
        'save_stack': False}

DYNAMIC_PARAMETERS_TYPES = {'seconds': ['isi', 'pre_stim', 'stim', 'post_stim', 'frame_length', 'avgint_adaptation'],
        'voltage': ['ir_imaging', 'ir_waiting', 'ir_livefeed', 'flash_on', 'flash_off'],
        'channel': ['ir_channel', 'flash_channel', 'trigger_channel', 'trigger_out_channel'],
        'integer': ['repeats', 'biosyst_channel'],
        'string': ['suffix', 'biosyst_stimulus', 'flash_type'],
        'boolean': ['save_stack']}


DYNAMIC_PARAMETERS_HELP = {'isi': 'Inter stimulus intervali[s]',
        'repeats': 'How many times the protocol is repeated',
        'pre_stim': 'How long to image before the pulse [s]',
        'stim': 'Stimulus (step pulse) length [s]',
        'post_stim': 'How long to image after the pulse [s]',
        'frame_length': 'Exposure time / inter-frame interval',
        'ir_imaging': 'IR brightness during image acqusition',
        'ir_waiting': 'IR brightness when waiting ISI',
        'ir_livefeed': 'IR brightness while updating the live image',
        'flash_on': 'Flash brightness during stim',
        'flash_off':' Flash brightness during image acqustition',
        'ir_channel': 'NI channel for IR',
        'flash_channel': 'NI channel for Flash',
        'trigger_channel': 'Trigger recieve/in channel',
        'trigger_out_channel': 'Trigger send/out channel',
        'suffix': 'Tag added to the saved folders',
        'biosyst_stimulus': 'Override the square pulse by a biosyst stimulus',
        'biosyst_channel': 'Channel of the biosyst simulus',
        'avgint_adaptation': 'Time to show stimulus mean value before imaging [s]',
        'flash_type': '"square" or sinelogsweep',
        'save_stack': 'If true, save stack instead separate images'}


def getRightType(parameter_name, string_value):
    '''
    Convert user inputted string to correct parameter value based on
    DYNAMIC_PARAMETER_TYPES

    TODO    - channel checking, check that the channel is proper NI channel
    '''
    if parameter_name in DYNAMIC_PARAMETERS_TYPES['integer']:
        return int(string_value)

   
    if parameter_name in DYNAMIC_PARAMETERS_TYPES['seconds']:
        if string_value.startswith('[') and string_value.endswith(']'):
            seconds = ast.literal_eval(string_value)

            for s in seconds:
                if s < 0:
                    raise ValueError('Here time is required to be strictly positive.')
        else:
            seconds = float(string_value)
            if seconds < 0:
                raise ValueError('Here time is required to be strictly positive.')
        return seconds

    if parameter_name in  DYNAMIC_PARAMETERS_TYPES['voltage']:
        if string_value.startswith('[') and string_value.endswith(']'):
            voltages = ast.literal_eval(string_value)
            for voltage in voltages:
                if not -10<=voltage<=10:
                    raise ValueError('Voltage value range -10 to 10 V exceeded.')
            return voltages
        else:
            voltage = float(string_value)
            if not -10<=voltage<=10:
                raise ValueError('Voltage value range -10 to 10 V exceeded.')
            return voltage

    if parameter_name in  DYNAMIC_PARAMETERS_TYPES['channel']:
        if type(string_value) == type(''):
            if string_value.startswith('[') and string_value.endswith([']']):
                return ast.literal_eval(string_value)
            else:
                return string_value
    
    if parameter_name in DYNAMIC_PARAMETERS_TYPES['string']:
        return str(string_value)

    if parameter_name in DYNAMIC_PARAMETERS_TYPES['boolean']:
        if string_value.lower() == 'true':
            return True
        elif string_value.lower() == 'false':
            return False
        else:
            raise ValueError('Boolean falue has to be either "True" or "False"')

    raise NotImplementedError('Add {} correctly to DYNAMIC_PARAMETER_TYPES in dynamic_parameters.py')



def load_parameters(fn):
    '''
    Loading imaging parameters, saved as a json file.
    '''
    with open(fn, 'r') as fp:
        data = json.load(fp)
    return data


def save_parameters(fn, parameters):
    '''
    Loading imaging parameters, saved as a json file.
    '''
    with open(fn, 'w') as fp:
        json.dump(parameters, fp)



class ParameterEditor:
    '''
    Dictionary editor on command line with ability to load and save presets.
    '''
    def __init__(self, dynamic_parameters, locked_parameters={}):
        '''
        dynamic_parameters      Dictionary of the dynamic imaging parameters.
        '''
        self.dynamic_parameters = dynamic_parameters
        self.parameter_names = sorted(self.dynamic_parameters.keys())

        self.presets_savedir = os.path.join(PUPILDIR, 'presets')
        self.presets = self.load_presets(self.presets_savedir)

        self.locked_parameters = locked_parameters

    
    def load_presets(self, directory):
        
        presets = {}
        
        if os.path.isdir(directory):
            files = [os.path.join(directory, fn) for fn in os.listdir(directory)]
        else:
            files = []

        for afile in files:
            try:
                preset = load_parameters(afile)
            except:
                print("Couldn't load preset {}".format(afile))
                continue
            
            # If older files lack some parameters, use default parameters
            for key in self.parameter_names:
                if not key in preset:
                    preset[key] = DEFAULT_DYNAMIC_PARAMETERS[key]
                    
            presets[os.path.basename(afile)] = preset

        return presets


    def print_preset(self, preset):
        '''
        Prints the current parameters and the help strings.
        '''
                 
        parameter_names = sorted(self.dynamic_parameters.keys())

        print()
        
        print('{:<20} {:<40} {}'.format('PARAMETER NAME', 'VALUE', 'DESCRIPTION'))
        for parameter in parameter_names:
            if parameter in self.locked_parameters:
                lck = ' (LOCKED to {})'.format(self.locked_parameters[parameter])
            else:
                lck = ''
            print('{:<20} {:<40} {}'.format(parameter, str(preset[parameter])+lck,
                DYNAMIC_PARAMETERS_HELP[parameter]))
        print()


    def getModified(self):
        '''
        Ask user to edit the parameters until happy and then return
        the parameters.
        '''
        
        while True:
            print('MODIFYING IMAGING PARAMETERS')
            self.print_preset(self.dynamic_parameters)
            parameter = input('Parameter name or (list/save/load) (Enter to continue) >> ')
            
            # If breaking free
            if parameter == '':
                break
            

            self.presets = self.load_presets(self.presets_savedir)

            
            # If saving preset
            if parameter.lower() == 'save':
                name = input('Save current parameters under preset name (if empty == suffix) >> ')
                if name == '' and self.dynamic_parameters['suffix'] != '':
                    name = self.dynamic_parameters['suffix']
                
                if os.path.isdir(PUPILDIR):
                    os.makedirs(self.presets_savedir, exist_ok=True)
                    save_parameters(os.path.join(self.presets_savedir, name), self.dynamic_parameters)
                else:
                    print('Saving the preset failed, {} does not exist'.format(PUPILDIR))

                continue        

            if parameter.lower() == 'list':
                if self.presets is {}:
                    print('There are no existing presets!')
                else:
                    print('These are the existing presets:')
                    for preset in self.presets.keys():
                        print('  '+preset)
                    print('')
                continue

            if parameter.lower() == 'load':
                # If parameter is actually a preset
                
                while True:
                    
                    preset_names = sorted(self.presets.keys())

                    for i, name in enumerate(preset_names):
                        print('{}) {}'.format(i+1, name))
                    
                    sel = input('>> ')

                    try:
                        to_load = preset_names[int(sel)-1]
                        break
                    except:
                        print('Invalid preset.')

                parameter = to_load

            if parameter in self.presets.keys():
                self.print_preset(self.presets[parameter])
                
                confirm = input('Load this (y/n)>> ').lower()
                if confirm and confirm[0] == 'y':
                    self.dynamic_parameters = self.presets[parameter]
                else:
                    print('Answer not yes, loading of the preset cancelled')
                continue
            
    

            try:
                self.dynamic_parameters[parameter]
            except KeyError:
                print('Invalid input, not a parameter name')
                time.sleep(1)
                continue

            while True:

                value = input('Value for {} >> '.format(parameter))
                
                if value == '':
                    break

                try:
                    value = getRightType(parameter, value)
                except ValueError as e:
                    print(str(e))
                    print('Could not convert the value to right type. Try againg or Enter to skip.')
                    continue

                self.dynamic_parameters[parameter] = value
                break

        return {**self.dynamic_parameters, **self.locked_parameters}


def getModifiedParameters(**kwargs):
    '''
    Take in the DEFAULT parameters in the beginning of this code file
    and let the user modify them using text based ParameterEditor
    '''
    editor = ParameterEditor(DEFAULT_DYNAMIC_PARAMETERS, **kwargs)
    return editor.getModified()



if __name__ == "__main__":
    print(getModifiedParameters())






import os

import numpy as np
import scipy.signal

from biosystfiles import extract as bsextract

class StimulusBuilder:
    '''
    Get various stimulus waveforms
    - to the stimulus LED
    - and on pulse for illumination LED
    - and square wave for triggering the camera.
    
    
    '''

    def __init__(self, stim_time, prestim_time, poststim_time, frame_length,
            stimulus_intensity, illumination_intensity, fs,
            stimulus_finalval=0, illumination_finalval=0,
            wtype='square'):
            '''
            stim_time               The time stimulus LED is on
            prestim_time            The time the camera is running and illumination is on before the stimulus
            poststim_time           The time the camera is running and illumination is on after the stimulus
            stimulus_intensity      From 0 to 1, the brightness of the stimulus
            illumination_intensity  From 0 to 1, the brightness of the illumination lights
            wtype                   "square" or "sinelogsweep" or "squarelogsweep"

            '''

            self.stim_time = stim_time
            self.prestim_time = prestim_time
            self.poststim_time = poststim_time
            self.frame_length = frame_length
            self.stimulus_intensity = stimulus_intensity
            self.illumination_intensity = illumination_intensity
            self.fs = fs
            self.stimulus_finalval = stimulus_finalval
            self.illumination_finalval = illumination_finalval

            self.wtype = wtype

            self.N_frames = int(round((stim_time+prestim_time+poststim_time)/frame_length))

            self.overload_stimulus = None
            


    def overload_biosyst_stimulus(self, fn, channel=0):
        '''
        Loads a Biosyst stimulus that gets returned then at
        get_stimulus_pulse instead.

        Returns the overload stimulus and new fs
        '''
        ffn = os.path.join('biosyst_stimuli', fn)
        self.overload_stimulus, self.fs = bsextract(ffn, channel)
        self.overload_stimulus = self.overload_stimulus.flatten()
        print(self.overload_stimulus.shape)
        print(np.max(self.overload_stimulus))

        return self.overload_stimulus, self.fs
    
    def get_stimulus_pulse(self):
        '''
        Constant value pulse

                _________stimulus_intensity
                |       |
        ________|       |__________
        prestim   stim    poststim
        '''

        if self.overload_stimulus is not None:
            return self.overload_stimulus

        N0_samples = int(self.prestim_time*self.fs)
        N1_samples = int(self.stim_time*self.fs)
        N2_samples = int(self.poststim_time*self.fs)
        
        if self.wtype == 'square':
            stimulus = np.concatenate( (np.zeros(N0_samples), np.ones(N1_samples), np.zeros(N2_samples)) )
        elif 'logsweep' in self.wtype:
            try:
                wtype, f0, f1 = self.wtype.split(',')
                f0 = float(f0)
                f1 = float(f1)
            except:
                print("Doing logsweep from 0.5 Hz to 100 Hz")
                f0=0.5
                f1=100
                wtype = self.wtype
            
            times = np.linspace(0, self.stim_time, N1_samples)
            active = scipy.signal.chirp(times, f0=f0, f1=f1, t1=self.stim_time, phi=-90, method='logarithmic')
            
            if wtype == 'squarelogsweep':
                active[active>0] = 1
                active[active<0] = -1
            elif wtype == '3steplogsweep':
                cstep = np.sin(np.pi/4)
                active[np.abs(active) <= cstep] = 0
                active[active > cstep] = 1
                active[active < -cstep] = -1
                
            elif wtype == 'sinelogsweep':
                pass
            else:
                raise ValueError('Unkown flash_type'.format(wtype))
                
            # Join with pre and post 0.5 values
            # and move and scale between 0 and 1 (from - 1 and 1)
            stimulus = np.concatenate( (np.ones(N0_samples)/2, (active+1)/2, np.ones(N2_samples)/2) )
            
        else:
            raise ValueError('Invalid wtype given, has to be "square" or "sinelogsweep" or "3steplogsweep"')

        stimulus = self.stimulus_intensity * stimulus

   
        stimulus[-1] = self.stimulus_finalval
        
        return stimulus



    def get_illumination(self):
        '''
        Returns 1D np.array.
        '''
        illumination = np.ones( int((self.stim_time+self.prestim_time+self.poststim_time)*self.fs) )
        illumination = self.illumination_intensity * illumination

        illumination[-1] = self.illumination_finalval
        
        return illumination



    def get_camera(self):
        '''
        Get square wave camera triggering stimulus.
        
        Returns 1D np.array.
        '''
        
        samples_per_frame = int(frame_length * fs /2)
        
        camera = np.concatenate( ( np.ones((samples_per_frame, self.N_frames)), np.zeros((samples_per_frame, self.N_frames)) ) ).T.flatten()
        camera = 5*camera
    
        camera[-1] = 0

        return camera



'''
Image accusition using Micro-Manager's Python (2) bindings (Camera class)
and a server program (CameraServer class).

On Windows, MM builds come compiled with Python 2 support only, so in this solution
there is a Python 2 server program that controls the camera and image saving
and then the client end that can be run with Python 3.
'''

import os
import time
import datetime
import socket
import threading
import multiprocessing

import MMCorePy
import tifffile
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.animation import FuncAnimation
from matplotlib.widgets import RectangleSelector

import camera_communication as cac
from camera_communication import SAVING_DRIVE

DEFAULT_SAVING_DIRECTORY = "D:\imaging_data"



class ImageShower:
    '''
    Showing images on the screen on its own window.

    In future, may be used to select ROIs as well to allow
    higher frame rate imaging / less data.
    
    ------------------
    Working principle
    ------------------
    Image shower works so that self.loop is started as a separate process
    using multiprocessing library
    
    -------
    Methods
    -------
    self.loop       Set this as multiprocessing target

    '''
    def __init__(self):
        self.fig = plt.figure()
        self.ax = self.fig.add_subplot(111)
        self.close = False

        #self.cid = self.fig.canvas.mpl_connect('key_press_event', self.callbackButtonPressed)
        
        self.image_brightness = 0
        self.image_maxval = 1

        self.selection = None

        self.image_size = None

    def callbackButtonPressed(self, event):
        
        if event.key == 'r':
            self.image_maxval -= 0.05
            self._updateImage(strong=True)
        
        elif event.key == 't':
            self.image_maxval += 0.05
            self._updateImage(strong=True)

            

    def __onSelectRectangle(self, eclick, erelease):
        
        # Get selection box coordinates and set the box inactive
        x1, y1 = eclick.xdata, eclick.ydata
        x2, y2 = erelease.xdata, erelease.ydata
        #self.rectangle.set_active(False)
        
        x = int(min((x1, x2)))
        y = int(min((y1, y2)))
        width = int(abs(x2-x1))
        height = int(abs(y2-y1))
        
        self.selection = [x, y, width, height]
        
    def _updateImage(self, i):
        
        data = None
        while not self.queue.empty():
            # Get the latest image in the queue
            data = self.queue.get(True, timeout=0.01)
        if data is None:
            return self.im, ''
        elif data == 'close':
            self.close = True
            return self.im, ''

        if self.selection and data.size != self.image_size:
            self.selection = None

        if self.selection:
            x,y,w,h = self.selection
            if w<1 or h<1:
                # If selection box empty (accidental click on the image)
                # use the whole image instead
                inspect_area = data
            else:
                inspect_area = data[y:y+h, x:x+w]
        else:
            inspect_area = data
        
        
        per95 = np.percentile(inspect_area, 95)
        data = np.clip(data, np.percentile(inspect_area, 5), per95)
        
        data = data - np.min(data)
        data_max = np.max(data)
        data = data.astype(float)/data_max

        self.image_size = data.size
       
        
        self.im.set_array(data)
        self.fig.suptitle('Selection 95th percentile: {}'.format(per95), fontsize=10)
        text = ''
        return self.im, text
           
         
    def loop(self, queue):
        '''
        Runs the ImageShower by reading images from the given queue.
        Set this as a multiprocessing target.

        queue           Multiprocessing queue with a get method.
        '''
        self.queue = queue
        self.rectangle = RectangleSelector(self.ax, self.__onSelectRectangle, useblit=True)
        
        image = queue.get()
        self.im = plt.imshow(1000*image/np.max(image), cmap='gray', vmin=0, vmax=1, interpolation='none', aspect='auto')
        self.ani = FuncAnimation(plt.gcf(), self._updateImage, frames=range(100), interval=5, blit=False)

        plt.show(block=False)
        
        while not self.close:
            plt.pause(1)


class DummyCamera:
    '''
    A dummy camera class, used when unable to load the real Camera class
    due to camera being off or something similar.
    '''
    def acquire_single(self, save, subdir):
        pass
    def acquire_series(self, exposure_time, image_interval, N_frames, label, subdir, trigger_direction):
        pass
    def save_images(images, label, metadata, savedir):
        pass
    def set_saving_directory(self, saving_directory):
        pass
    def set_binning(self, binning):
        pass
    def save_description(self, filename, string):
        pass
    def close(self):
        pass


class Camera:
    '''
    Controlling ORCA FLASH 4.0 camera using Micro-Manager's
    Python (2) bindings.
    '''

    def __init__(self, saving_directory=DEFAULT_SAVING_DIRECTORY):

        self.set_saving_directory(saving_directory)
        
        self.mmc = MMCorePy.CMMCore() 
        self.mmc.loadDevice('Camera', 'HamamatsuHam', 'HamamatsuHam_DCAM')
        self.mmc.initializeAllDevices()
        self.mmc.setCameraDevice('Camera')
            
        self.settings = {'binning': '1x1'}
        
        self.mmc.prepareSequenceAcquisition('Camera')
        #self.mmc.setCircularBufferMemoryFootprint(4000)
        self.live_queue= False

        self.shower = ImageShower()

        # Description file string
        self.description_string = ''

        self.save_stack = False



    def acquire_single(self, save, subdir):
        '''
        Acquire a single image.

        save        'True' or 'False'
        subdir      Subdirectory for saving
        '''
        
        exposure_time = 0.01
        binning = '2x2'

        self.set_binning(binning)
        self.mmc.setExposure(exposure_time*1000)

        start_time = str(datetime.datetime.now())
 
        self.mmc.snapImage()
        image = self.mmc.getImage()
        
        if not self.live_queue:
            self.live_queue = multiprocessing.Queue()
            self.live_queue.put(image)
            
            self.livep = multiprocessing.Process(target=self.shower.loop, args=(self.live_queue,))
            self.livep.start()
            
        self.live_queue.put(image)

        if save == 'True':
            metadata = {'exposure_time_s': exposure_time, 'binning': binning, 'function': 'acquireSingle', 'start_time': start_time}

            save_thread = threading.Thread(target=self.save_images,args=([image],'snap_{}'.format(start_time.replace(':','.').replace(' ','_')), metadata,os.path.join(self.saving_directory, subdir)))
            save_thread.start()



    def acquire_series(self, exposure_time, image_interval, N_frames, label, subdir, trigger_direction):
        '''
        Acquire a series of images

        exposure_time       How many seconds to expose each image
        image_interval      How many seconds to wait in between the exposures
        N_frames            How many images to take
        label               Label for saving the images (part of the filename later)
        subdir
        trigger_direction   "send" (camera sends a trigger pulse when it's ready) or "receive" (camera takes an image for every trigger pulse)
        '''

        exposure_time = float(exposure_time)
        image_interval = float(image_interval)
        N_frames = int(N_frames)
        label = str(label)

        print "Now aquire_series with label " + label
        print "- IMAGING PARAMETERS -"
        print " exposure time " + str(exposure_time) + " seconds"
        print " image interval " + str(image_interval) + " seconds"
        print " N_frames " + str(N_frames)
        print "- CAMERA SETTINGS"

        self.set_binning('2x2')
        print " Pixel binning 2x2"

        if trigger_direction == 'send':
            print " Camera sending a trigger pulse"
            self.mmc.setProperty('Camera', "OUTPUT TRIGGER KIND[0]","EXPOSURE")
            self.mmc.setProperty('Camera', "OUTPUT TRIGGER POLARITY[0]","NEGATIVE")
        elif trigger_direction== 'receive':
            print " Camera recieving / waiting for a trigger pulse"
            self.mmc.setProperty('Camera', "TRIGGER SOURCE","EXTERNAL")
            self.mmc.setProperty('Camera', "TriggerPolarity","POSITIVE")
        else:
            raise ValueError('trigger_direction has to be {} or {}, not {}'.format('receive', 'send', trigger_direction))

        
        print "Circular buffer " + str(self.mmc.getCircularBufferMemoryFootprint()) + " MB"

        self.mmc.setExposure(exposure_time*1000)


        self.wait_for_client()
        

        start_time = str(datetime.datetime.now())
        self.mmc.startSequenceAcquisition(N_frames, image_interval, False)
        
        
        while self.mmc.isSequenceRunning():
            time.sleep(exposure_time)

        images = []

        for i in range(N_frames):
            while True:
                try:
                    image = self.mmc.popNextImage()
                    break
                except MMCorePy.CMMError:
                    time.sleep(exposure_time)
                
            images.append(image)
            
            
        metadata = {'exposure_time_s': exposure_time, 'image_interval_s': image_interval,
                    'N_frames': N_frames, 'label': label, 'function': 'acquireSeries', 'start_time': start_time}
        metadata.update(self.settings)

        save_thread = threading.Thread(target=self.save_images, args=(images,label,metadata,os.path.join(self.saving_directory, subdir)))
        save_thread.start()
        
        self.mmc.setProperty('Camera', "TRIGGER SOURCE","INTERNAL")
        print('acquired')

    
    def save_images(self, images, label, metadata, savedir):
        '''
        Save given images as grayscale tiff images.
        '''
        if not os.path.isdir(savedir):
            os.makedirs(savedir)

        if self.save_stack == False:
            # Save separate images
            for i, image in enumerate(images):
                fn = '{}_{}.tiff'.format(label, i)
                tifffile.imsave(os.path.join(savedir, fn), image, metadata=metadata)
        else:
            # Save a stack
            fn = '{}_stack.tiff'.format(label)
            tifffile.imsave(os.path.join(savedir, fn), np.asarray(images), metadata=metadata)
        
        self.save_description(os.path.join(savedir, 'description'), self.description_string, internal=True)


    def set_saving_directory(self, saving_directory):
        '''
        Sets where the specimen folders are saved and if the directory
        does not yet exist, creates it.
        '''
        saving_directory = os.path.join(SAVING_DRIVE, saving_directory)
        if not os.path.isdir(saving_directory):
            os.makedirs(saving_directory)
            
        self.saving_directory = saving_directory


    def set_save_stack(self, boolean):
        '''
        If boolean == "True", save images as stacks instead of separate images.
        '''
        if boolean == 'True':
            self.save_stack = True
        elif boolean == 'False':
            self.save_stack = False
        else:
            print("Did not understand wheter to save stacks. Given {}".format(boolean))

    def set_binning(self, binning):
        '''
        Binning '2x2' for example.
        '''
        if not self.settings['binning'] == binning:
            self.mmc.setProperty('Camera', 'Binning', binning)
            self.settings['binning'] =  binning

    def set_roi(self, x,y,w,h):
        '''
        In binned pixels
        roi     (x,y,w,h) or None
        '''
        x = int(x)
        y = int(y)
        w = int(w)
        h = int(h)
        
        if w == 0 or h==0:
            self.mmc.clearROI()
        else:
            self.mmc.setROI(x,y,w,h)


    def save_description(self, specimen_name, desc_string, internal=False):
        '''
        Allows saving a small descriptive text file into the main saving directory.
        Filename should be the same as the folder where it's saved.

        Appends to the previous file.

        specimen_name           DrosoM42 for example, name of the specimen folder
        desc_string             String, what to write in the file
        internal                If true, specimen_name becomes filename of the file
        '''
        if internal:
            fn = specimen_name
        else:
            fn = os.path.join(self.saving_directory, specimen_name, specimen_name)
        
        # Check if the folder exists
        if not os.path.exists(os.path.dirname(fn)):
            #raise OSError('File {} already exsits'.format(fn))
            os.makedirs(os.path.dirname(fn))
        
        with open(fn+'.txt', 'w') as fp:
            fp.write(desc_string)

        print "Wrote file " + fn+'.txt'
        
        
        self.description_string = desc_string


    def close(self):
        self.live_queue.put('close')
        self.lifep.join()

    def wait_for_client(self):
        pass


class CameraServer:
    '''
    Camera server listens incoming connections and
    controls the camera through Camera class
    '''
    def __init__(self):

        PORT = cac.PORT
        HOST = ''           # This is not cac.SERVER_HOSTNAME, leave empty

        self.running = False

        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.bind((HOST, PORT))
        self.socket.listen(1)

        try:
            self.cam = Camera()
            self.cam.wait_for_client = self.wait_for_client
        except Exception as e:
            print e
            print "Using DUMMY camera instead"
            self.cam = DummyCamera()
        
        self.functions = {'acquireSeries': self.cam.acquire_series,
                          'setSavingDirectory': self.cam.set_saving_directory,
                          'acquireSingle': self.cam.acquire_single,
                          'saveDescription': self.cam.save_description,
                          'set_roi': self.cam.set_roi,
                          'set_save_stack': self.cam.set_save_stack,
                          'ping': self.ping,
                          'exit': self.stop}

    def ping(self, message):
        print message



    def wait_for_client(self):
        '''
        Waits until client confirms that it is ready
        '''
        conn, addr = self.socket.accept()
        string = ''
        while True:
            data = conn.recv(1024)
            if not data: break
            string += data
        conn.close()
        print "Client ready"
        


    def run(self):
        '''
        Loop waiting for incoming connections.
        Each established connection can give one command and then the connection
        is closed.
        '''
        self.running = True
        while self.running:
            conn, addr = self.socket.accept()
            string = ''
            while True:
                data = conn.recv(1024)
                if not data: break
                string += data
            
            conn.close()
            print('Recieved command "'+string+'" at time '+str(time.time()))
            if string:
                func, parameters = string.split(';')
                parameters = parameters.split(':')
                target=self.functions[func](*parameters)
            
    def stop(self, placeholder):
        '''
        Stop running the camera server.
        '''
        self.camera.close()
        self.running = False


def test_camera():
    cam = Camera()
    images = cam.acquireSeries(0.01, 1, 5, 'testing')
    
    for image in images:
        plt.imshow(image, cmap='gray')
        plt.show()



def run_server():
    '''
    Running the server.
    '''
    cam_server = CameraServer()
    cam_server.run()
            
        
if __name__ == "__main__":
    run_server()



'''
Places where to save and load temporary files.
'''

import os
import platform

CODE_ROOTDIR = os.path.dirname(os.path.realpath(__file__))
USER_HOMEDIR = os.path.expanduser('~')

if platform.system() == "Windows":
    PUPILDIR = os.path.join(USER_HOMEDIR, 'GonioImsoft')
else:
    PUPILDIR = os.path.join(USER_HOMEDIR, '.gonioimsoft')
